#!/usr/bin/python
# -*- coding: utf-8 -*-

__author__ = 'lovro'
import os
from datetime import datetime as dt
from datetime import timedelta
from time import gmtime, mktime


# test
from tempfile import mkstemp
from shutil import move
from os import remove, close

# http requests
import json
import requests

# appdirs
import appdirs


class Parser:
    user_char = ''

    BASE_URL = 'http://dragon.foi.hr:5000/mw/api/v1/'

    latest_timestamp = ''
    timestamps_db_file = 'timestamps.db'
    
    app_name = 'ModelMMORPG'
    app_author = 'AILabFOI'
    template = '''// WARNING
// This file contains sensitive info about timestamps and please DO NOT TOUCH it.
// Do not try to modify od add something to this file at any cost !
#Battle 2015-01-01 00:00:00
#Debug 2015-01-01 00:00:00
#General 2015-01-01 00:00:00
#Party 2015-01-01 00:00:00
#Trade 2015-01-01 00:00:00
config 2015-01-01 00:00:00
'''

    def __init__(self):
        datadir = appdirs.user_data_dir( self.app_name, self.app_author )
        if not os.path.exists( datadir ):
            os.makedirs( datadir )
        self.timestamps_db_file = os.path.join( datadir, self.timestamps_db_file )
        

    def check_log_file_exists(self, path):
        """
        Method that checks if specific log file even exists
        Returns boolean
        Also recreates timestamp db file if accidentally deleted
        """
        if not os.path.isfile( self.timestamps_db_file ):
            f = open( self.timestamps_db_file, 'w' )
            f.write( self.template )
            f.close()
        if os.path.isfile(path):
            return True
        else:
            return False

    def parse_log_timestamp(self, line):
        """
        Method that parses line in current log file
        Returns timestamp
        """
        # TMW uses UTC time on the server
        i = self.get_utc()
        hours = int(line.split(']')[0].split('[')[1].split(':')[0])
        minutes = int(line.split(']')[0].split('[')[1].split(':')[1])
        timestamp = dt(int(i.year), int(i.month), int(i.day), hours, minutes)
        return timestamp

    def parse_db_timestamp(self, line):
        """
        Method that parses line for database file to get timestamp
        Returns timestamp
        """
        year = line.split()[1].split('-')[0]
        month = line.split()[1].split('-')[1]
        day = line.split()[1].split('-')[2]
        hours = line.split()[2].split(':')[0]
        minutes = line.split()[2].split(':')[1]

        return dt(int(year), int(month), int(day), int(hours), int(minutes))


    def parse_log_speaker(self, line):
        """
        Method that parses line in current log file and parses speaker name
        Returns string
        """
        try:
            return line.split()[1].split(":")[0]
        except:
            return ''

    pass

    def parse_log_text_data(self, line):
        """
        Method that parses users text data
        returns string
        """
        try:
            return line.split(':')[2]
        except:
            return line.split(':')[1]

    def parser_log_whisper_text(self):
        """
        This method is specialized form of parse_log_text_data method, this one is specifically designed to be used
         when parsing users log
        """
        pass

    def check_timestamp_exists_db(self, category_type):
        """
        Method that only checks if specific category exists in timestamps.txt
        """
        searchfile = open(self.timestamps_db_file, "r")
        for line in searchfile:
            if line[0] == '/':
                continue
            if category_type in line:
                searchfile.close()
                return True

        return False

    def compare_with_latest_timestamp(self, category_type, path):
        """
        Method that first checks ih log timestamp exists, it not it creates one and then
        compares timestamps.txt from relevant log file and timestamps.txt and returns logical expression
        """
        if self.check_timestamp_exists_db(category_type.split('.')[0]):
            #print "comparing " + str(self.get_latest_timestamp_file(path)) + " and " + str(self.get_timestamp_db(category_type.split('.')[0]))

            if self.get_latest_timestamp_file(path) > self.get_timestamp_db(category_type.split('.')[0]):
                # create log which will be sent to server
                return True
            else:
                # there is no change
                return False


    def get_latest_timestamp_file(self, path):
        """
        Takes path to log file as input param
        Method that returns latest timestamp from current log file
        Returns timestamp/string
        """
        searchfile = open(path, "r")
        for line in searchfile:
            pass
        self.latest_timestamp = self.parse_log_timestamp(line)
        return self.latest_timestamp

    def get_timestamp_db(self, category_type):
        """
        Method that returns relevant timestamp from timestamps.txt.txt
        Returns timestamp/string
        """

        searchfile = open(self.timestamps_db_file, "r")
        for line in searchfile:
            if line[0] == '/':
                continue
            if category_type in line:
                try:
                    time = line.split(':')[1]
                    # There is already something written so we proceed
                    searchfile.close()
                    return self.parse_db_timestamp(line)
                except:
                    # first run, we need to save latest and to not to go to recursion we will search again
                    searchfile.close()
                    if self.set_timestamp_db(category_type, self.latest_timestamp):
                        searchfile = open(self.timestamps_db_file, "r")
                        for line in searchfile:
                            if line[0] == '/':
                                continue
                            if category_type in line:
                                return self.parse_db_timestamp(line)

    def set_timestamp_db(self, category_type, timestamp):
        """
        This method saves new timestamp time to timestamps.txt file
        """
        category = category_type

        searchfile = open(self.timestamps_db_file, "r+")
        for line in searchfile:
            if category_type in line:
                searchfile.close()
                self.replace(self.timestamps_db_file, line, category + " " + str(timestamp) + "\n")
                return True

        searchfile.close()
        return False


    def replace(self, file_path, pattern, subst):
        # Create temp file
        """
        Method that replaces specific line in txt file
        :param file_path:
        :param pattern:
        :param subst:
        """
        try:
            fh, abs_path = mkstemp()
            new_file = open(abs_path, 'w')
            old_file = open(file_path)
            temp_template = []
            for line in old_file:
                new_file.write(line.replace(pattern, subst))
                temp_template.append( line )
            self.template = '\n'.join( temp_template )
            # close temp file
            new_file.close()
            close(fh)
            old_file.close()
            # Remove original file
            remove(file_path)
            # Move new file
            ok = False
            while not ok:
                try:
                    move(abs_path, file_path)
                    ok = True
                except:
                    pass
        except Exception, e:
            print str(e)

    def send_request(self, url_path, request, log_type):

        """
        Method that sends json request to server and if everything is OK then we write new time to timestamps.txt
        :param url_path:
        :param request:
        """
        try:
            #print request
            payload = json.dumps(request, ensure_ascii=True) #False
            headers = {'content-type': 'application/json'}
            r = requests.post(self.BASE_URL + url_path, data=payload, headers=headers)
            #print log_type + "  " + self.BASE_URL + url_path + "   " + payload
            try:
                data = r.json()
            except Exception, e:
                data = r.json
            #print data
            if data["status"] == "200":
                if self.set_timestamp_db(log_type, self.get_utc()):
                    print log_type + " saved"
                else:
                    print "failed writing timestamps"

        except Exception, e:
            print e
    
    def get_utc( self ):
        ''' Returns utc'''
        return dt.utcnow().replace( microsecond=0 )

